# Reading UIUC Financial Statements with pdf2data

The script [read-uiuc-fin-statement](read-uiuc-fin-statement) aims to read
financial statements made available through the `my.dept` system.  Possibly,
these are the same as the PDFs generated by My UI Financials and "EDDIE", but I
really don't know. In particular, it is designed to read the

* Operating Ledger Transaction Statement
* Payroll Labor Distribution

reports. I think those contain all the details, and the other ones are just
summaries of these, but I'd be curious to hear if that's incorrect.

It works around the (at least as far as I know) unavailability of bulk export
of financial information from UI systems. It (currently) works for me with a
1000-odd page PDF that comes out of the "Customize Combined Statements"
function, for all my active accounts, since "the beginning of time". As far as
I can tell, that includes the entire lifetime of
[XPACC](https://xpacc.illinois.edu/), to give you an idea of robustness.

So you run
```
read-uiuc-fin-statement ~/Downloads/Custom_Statements.pdf.pdf  # (sic!)
```
and the information from the PDF will be added to the SQLite3 file.  By
default, the tool will write `statements.sqlite3`.  If you run the tool twice
on the same input PDF file, you'll get duplicate entries, so don't do that.
(I'd be happy to take a patch!)

It is safe to ^C abort the tool. It will either finish completely or make no
modifications. (Yay transactions.)

If you are lucky, at the end of the process you will be left with a SQLite3
file containing the information from one or more giant PDFs. But you should not be
confused: Despite being (hopefully) expedient, this is a hilarious, absurd
process. Realize that this data already exists in a database somewhere! Why is it
being extracted out of a PDF by a tool of questionable reliability? The tool
generally tries to be somewhat careful and error out if it is confused, but if
you make crucial business decisions based on data generated by this tool, and
that data turns out to be wrong, you know who to blame. (Not me.)

Once you have the file, I highly recommend
[SQLiteBrowser](https://sqlitebrowser.org/) for analyzing it. You'll need to
know a bit of [SQL](https://en.wikipedia.org/wiki/SQL) to drive it. If you
don't, simply try and extrapolate from the queries below. You'll be right most
of the time. Alternatively, I hear the CS department has a course. :)

Here are some queries I've found useful:

*   What accounts do I have?

    ```sql
    select * from account;
    ```

*   How much money have I spent on my startup/ICR account? (`2` is the account ID of my
    startup account.)

    ```sql
    select sum(actual) from trans where in_account_id=2;
    ```

*   How much money has arrived from a funding agency? (3 is an NSF grant account for me)

    ```sql
    select sum(actual) from trans where in_account_id=3 and descr like '%Revenue%';
    ```

*   Who has been paid how much out of account X?

    ```sql
    select name, sum(amount) from payroll where in_account_id=2 group by name order by name;
    ```

*   Who has been paid how much out of account X?
    (broken down by salary, benefits, etc.)

    ```sql
    select name, account_descr, sum(amount)
    from payroll where in_account_id=3
    group by name, account_descr
    order by name, account_descr;
    ```

*   Out of which accounts (and how much) has RA `Lastname` ever been paid?
    (broken down by salary, benefits)

    ```sql
    select fund_descr, account_descr, sum(amount)
    from payroll inner join account on payroll.in_account_id = account.id where name like '%Lastname%'
    group by fund_descr, account_descr
    order by fund_descr, account_descr
    ;
    ```


*   What types of things have my accounts been charged for (and how much)?
    (excluding XPACC)

    ```sql
    select account_descr, sum(actual) from trans
        where in_account_id <> 5 and actual <> 0
        group by account_descr
        order by account_descr
        ;
    ```

*   How much have I spent on IT equipment? (not counting XPACC)

    ```sql
    select sum(actual) from trans where account_descr like 'NC IT Equip%' and actual <> 0 and in_account_id <> 5;
    ```

*   How much have I spent on travel? (myself and students, excluding XPACC)

    ```sql
    select sum(actual) from trans where (
        account_descr like '%Trav%'
        or account_descr like '%Reimb%'
        or account_descr like '%Trvl%'
        or account_descr like '%Veh%'
        ) and account_descr not like '%Non-Emp%'
        and in_account_id <> 5;
    ```

*   How much indirect cost have I paid to the university?  (excluding XPACC)

    ```sql
    select sum(actual) from trans
        where in_account_id <> 5 and actual <> 0
        and account_descr like '%Fac%' and  account_descr like '%Adm%'
        ;
    ```

*   For each account, compare 'revenue' with indirect costs:

    ```sql
    select
      fund_descr,
      case
        when (account_descr like '%Fac%' and  account_descr like '%Adm%') then 'F&A'
            else account_descr
      end as trans_type,
      sum(actual)
    from trans inner join account on trans.in_account_id = account.id
    where (
            actual <> 0
            and (
                account_descr like '%Oper Rev%'
                or trans_type == 'F&A'
            ))
    group by fund_descr,trans_type
    ```

*   Looking back over the last statement periods, show transactions above 100$ on a given account:

    ```sql
    select fy, period, date, account_descr, actual
    from trans
    where in_account_id = 3 and abs(actual) > 100
    order by fy desc, period desc
    ```

*   Looking back over the last statement periods, show payroll items on a given account:

    ```sql
    select fy, period, name, pay_period_begin, account_descr, amount
    from payroll
    where in_account_id = 3
    order by fy desc, period desc
    ```

*   Show a running total (vs date) of expenses on a given account. SQLiteBrowser has a handy function
    that lets you graph this information.

    ```sql
    select id, account_descr, date, actual,  sum(actual) over (order by date, id) as running
    from trans
    where in_account_id = 2 and actual <> 0 and account_descr not like '%Revenue%'
    order by date, id
    ;

If you find other useful queries, or if you have improvements to the tool, please fork this
project and submit a pull request!
